
#include "rtl_types.h"
#include "assert.h"
#include "types.h"
#include "rtl_utils.h"
#include "rtl_queue.h"
#include "rtl_errno.h"
#include "mbuf.h"


#include "rtl8651_tblDrvFwdLocal.h"
#include "rtl8651_tblDrvFwd.h"
#include "rtl8651_tblDrvProto.h"
#include "rtl8651_tblDrvLocal.h"
#include "rtl8651_alg_init.h"
#include "rtl8651_alg_battlenet.h"


#define MAX_SC_CLIENT 8
static uint32 clientIpList[MAX_SC_CLIENT]={0};

#define _RTL8651_L4_MAX_SC_ALIAS_BUF_SIZE 576
static uint8 *_rtl8651_l4_sc_alias_buf[2];

static int32 init_sc = 0;

int32 rtl8651_l4AliasHandleBattlenetInit(void *arg)
{	
	
	if(init_sc)
		return SUCCESS;

	_rtl8651_l4_sc_alias_buf[0] = (uint8 *)rtlglue_malloc(_RTL8651_L4_MAX_SC_ALIAS_BUF_SIZE);
	assert(_rtl8651_l4_sc_alias_buf[0]);
	
	_rtl8651_l4_sc_alias_buf[1] = (uint8 *)rtlglue_malloc(_RTL8651_L4_MAX_SC_ALIAS_BUF_SIZE);
	assert(_rtl8651_l4_sc_alias_buf[1]);
	init_sc = 1;
	return SUCCESS;
}


void  rtl8651_l4AliasHandleBattlenetReInit(void)
{
	int i;
	for(i=0;i<MAX_SC_CLIENT;i++) clientIpList[i]=0;
}



int32 rtl8651_l4AliasHandleBattlenetClientOutbound(struct rtl_pktHdr *pkthdrPtr, struct ip *pip,
			struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tb)
{
	struct rtl_pktHdr *indat = (struct rtl_pktHdr *)pkthdrPtr;
	struct rtl_mBuf *mbuf = indat->ph_mbuf;
	int8 *newPacket;
	struct ip *p_ip;
	int8  localip[4];
	int8	localip_len;
	int8  globalip[4];
	int8	globalip_len;
	int8 *sptr;
	int32 offset;
	



		
	if((alg_supports[pkthdrPtr->ph_dsid]&RTL8651_ALG_BATTLENET)==0) return SUCCESS;
	
	
	newPacket =  (int8 *)_rtl8651_l4_sc_alias_buf[0];
	
	
	offset = ((uint8 *)pip) - mbuf->m_data;
	if (offset < 0 || offset > 40 || tb == NULL)
		return FWDENG_USER_DROP;
	
	
	mBuf_copyToUserBuffer( mbuf, offset, mbuf->m_pkthdr->ph_len - offset, newPacket );	
	p_ip= (struct ip *)newPacket;	
	sptr = (int8 *)p_ip;
	





	localip[0]		= 	(tb->insideLocalIpAddr>>24)&0xff;
	localip[1] 	= 	(tb->insideLocalIpAddr>>16)&0xff;
	localip[2] 	= 	(tb->insideLocalIpAddr>>8)&0xff;
	localip[3] 	= 	(tb->insideLocalIpAddr)&0xff;
	
	globalip[0] 	= 	(tb->insideGlobalIpAddr>>24)&0xff;
	globalip[1] 	= 	(tb->insideGlobalIpAddr>>16)&0xff;
	globalip[2] 	= 	(tb->insideGlobalIpAddr>>8)&0xff;
	globalip[3] 	= 	(tb->insideGlobalIpAddr)&0xff;

	localip_len = 4;
	globalip_len = 4;
	
	if(pip->ip_p==IPPROTO_TCP)
	{
		struct tcphdr *th = NULL;
		uint8 *data;
		uint32   hlen, tlen, dlen;

		th = (struct tcphdr *) ((int8 *) pip + ((pip->ip_vhl &0xf) << 2));
		hlen =((pip->ip_vhl & 0xf) + (( th->th_off_x >>4)&0xf)) << 2;
		tlen = ntohs(pip->ip_len);
		dlen = tlen - hlen;	
		data = (uint8 *)(th + 1);
		sptr += hlen;
		
		if(data[0]==0xff)
		{
			if((dlen>23)&&((data[1]==0x50) || (data[1] == 0x09))) 
			{
				int i,j;
				
				for(i=0,j=0;i<dlen;i++,j++)
				{
					if ( memcmp(&data[i], localip, localip_len) ==0 )
					{

						rtl8651_memcpy(&sptr[j],globalip,globalip_len);
						i+=(localip_len-1);
						j+=(globalip_len-1);
					}
					else
					{
						sptr[j]=data[i];
					}
				}
				
				
				mBuf_copyToMbuf( mbuf, offset, tlen, newPacket );				
			}
				
		}
	}

	return SUCCESS;
}


int32 rtl8651_l4AliasHandleBattlenetClientInbound(struct rtl_pktHdr *pkthdrPtr, struct ip *pip,
			struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tb)
{

	struct rtl_pktHdr *indat = (struct rtl_pktHdr *)pkthdrPtr;
	struct rtl_mBuf *mbuf = indat->ph_mbuf;
	int8 *newPacket;
	struct ip *p_ip;
	int8  localip[4];
	int8	localip_len;
	int8  globalip[4];
	int8	globalip_len;







	int8 *sptr;
	int32 offset;

	
	localip[0]		= 	(tb->insideLocalIpAddr>>24)&0xff;
	localip[1] 	= 	(tb->insideLocalIpAddr>>16)&0xff;
	localip[2] 	= 	(tb->insideLocalIpAddr>>8)&0xff;
	localip[3] 	= 	(tb->insideLocalIpAddr)&0xff;
	
	globalip[0] 	= 	(tb->insideGlobalIpAddr>>24)&0xff;
	globalip[1] 	= 	(tb->insideGlobalIpAddr>>16)&0xff;
	globalip[2] 	= 	(tb->insideGlobalIpAddr>>8)&0xff;
	globalip[3] 	= 	(tb->insideGlobalIpAddr)&0xff;

	localip_len	= 4;
	globalip_len 	= 4;


	
	if((alg_supports[pkthdrPtr->ph_dsid]&RTL8651_ALG_BATTLENET)==0) return SUCCESS;

	newPacket =  (int8 *)_rtl8651_l4_sc_alias_buf[1];
	
	
	offset = ((uint8 *)pip) - mbuf->m_data;
	if (offset < 0 || offset > 40 || tb == NULL)
		return FWDENG_USER_DROP;
	
	
	mBuf_copyToUserBuffer( mbuf, offset, mbuf->m_pkthdr->ph_len - offset, newPacket );	
	p_ip= (struct ip *)newPacket;	
	sptr = (int8 *)p_ip;

	if(pip->ip_p==IPPROTO_TCP)
	{
		struct tcphdr *th = NULL;
		uint8 *data;
		uint32   hlen, tlen, dlen;

		th = (struct tcphdr *) ((int8 *) pip + ((pip->ip_vhl &0xf) << 2));
		hlen =((pip->ip_vhl & 0xf) + (( th->th_off_x >>4)&0xf)) << 2;
		tlen = ntohs(pip->ip_len);
		dlen = tlen - hlen;	
		data = (uint8 *)(th + 1);
		sptr += hlen;
		
		if(data[0]==0xff)
		{
			if((data[1]==0x09) || (data[1]==0x50))
			{
				int i;
				uint16 result_port = (tb->insideLocalIpAddr)&0xff;
				result_port = 60000 + result_port;
				for(i=0;i<MAX_SC_CLIENT;i++)
				{
					if(clientIpList[i]== tb->insideLocalIpAddr)  
					{
						break;
					}
					else if(clientIpList[i]==0)
					{
						clientIpList[i]= tb->insideLocalIpAddr;

						_rtl8651_addNaptConnection(TRUE, TRUE,UDP_FLOW, tb->insideLocalIpAddr, 6112, &tb->insideGlobalIpAddr, &result_port,tb->dstIpAddr,6112);
						rtl8651_addNaptServerPortMapping(FALSE,tb->insideGlobalIpAddr,result_port,tb->insideLocalIpAddr, 6112);

						break;
					}					
				}
#if 0				
				if (dlen>23)
				{
					
					
					
					for(i=0,j=0;i<dlen;i++,j++)
					{
						if ( memcmp(&data[i], globalip, globalip_len) ==0 )
						{

							int16 temp = 0;
							temp = data[i-2];

							glocalport_num = ((temp & 0xff)<<8 ) + data[i-1];

							if ((sc_tb = _rtl8651_findNaptConnectionByExtIpPort(FALSE, tb->insideGlobalIpAddr, glocalport_num, 0, 0)) != NULL)
							{
								localip_num = sc_tb->insideLocalIpAddr;
								localport_num = sc_tb->insideLocalPort;

								localip[0]		= 	(localip_num>>24)&0xff;
								localip[1] 	= 	(localip_num>>16)&0xff;
								localip[2] 	= 	(localip_num>>8)&0xff;
								localip[3] 	= 	(localip_num)&0xff;
								localport[0]	=	(localport_num>>8)&0xff;
								localport[1]	=	(localport_num)&0xff;
							}
							rtl8651_memcpy(&sptr[j], localip, localip_len);
							rtl8651_memcpy(&sptr[j-2], localport, localport_len);
							i+=(localip_len-1);
							j+=(globalip_len-1);
						}
						else
						{
							sptr[j]=data[i];
						}
					}
					
					
					mBuf_copyToMbuf( mbuf, offset, tlen, newPacket );
				}				
#endif				
			}
				
		}
	}

	return SUCCESS;
}


