


#include "assert.h"
#include "types.h"
#include "rtl_queue.h"
#include "rtl_errno.h"
#include "mbuf.h"
#include "rtl8651_tblDrvFwdLocal.h"
#include "rtl8651_tblDrvProto.h"
#include "rtl8651_tblDrvLocal.h"

#include "rtl8651_alg_init.h"
#include "rtl8651_alg_ftp.h"
#include "rtl8651_alg_h323.h"


#include "rtl8651_alg_dx7play.h"
#include "rtl8651_alg_pptp.h"

#include "rtl8651_alg_ipsec.h"
#include "rtl8651_alg_ymsg.h"
#include "rtl8651_alg_icq.h"
#include "rtl8651_alg_irc_dcc.h"
#include "rtl8651_alg_l2tp.h"
#include "rtl8651_alg_sip.h"
#include "rtl8651_alg_vdolive.h"
#include "rtl8651_alg_cs.h"
#include "rtl8651_alg_quake.h"
#include "rtl8651_alg_battlenet.h"
#include "rtl8651_alg_realaudio.h"
#include "rtl8651_alg_rtsp.h"
#include "rtl8651_alg_cuseeme.h"
#include "rtl8651_alg_tftp.h"
#include "rtl8651_tblDrvLocal.h"

uint32 alg_supports[RTL8651_MAX_DIALSESSION]={0};
uint32 alg_serverips[RTL8651_MAX_DIALSESSION][32]={{0}};
uint32 alg_qosqid[RTL8651_MAX_DIALSESSION][32]={{0}};
uint32 alg_qoshl[RTL8651_MAX_DIALSESSION][32]={{0}};
uint32 alg_qosRatio[RTL8651_MAX_DIALSESSION][32]={{0}};

struct rtl8651_algMaxSession algMaxSession={32,32,32};

uint32 rtl8651_getAlgStatus(void)
{
	return alg_supports[0];
}

uint32 rtl8651a_getAlgStatus(uint32 SessionID)
{
	uint32 dsid=_rtl8651_SessionIDToDSID(SessionID);
	if(dsid>=RTL8651_MAX_DIALSESSION)
		return 0;
	else
		return alg_supports[dsid];
}


uint32 rtl8651_flushAlgServerIn(void)
{
	rtl8651a_flushAlgServerIn(0);
	return SUCCESS;
}
	
uint32 rtl8651a_flushAlgServerIn(uint32 SessionID)
{
	uint32 dsid=_rtl8651_SessionIDToDSID(SessionID);
	ipaddr_t gip=_rtl8651_getExtIpAddrByDsid(dsid);
	

	
	rtl8651a_delNaptServerPortMapping(dsid,1,gip, 21,alg_serverips[dsid][RTL8651_ALG_FTP_IDX], 21);

	
	rtl8651a_delNaptServerPortMapping(dsid,1,gip, 1720,alg_serverips[dsid][RTL8651_ALG_NETMEETING_IDX], 1720);
	rtl8651a_delNaptServerPortMapping(dsid,1,gip, 1503,alg_serverips[dsid][RTL8651_ALG_NETMEETING_IDX], 1503);	

	
	rtl8651a_delNaptServerPortMapping(dsid,1,gip, 1723,alg_serverips[dsid][RTL8651_ALG_PPTP_IDX], 1723);


	
	rtl8651a_delNaptServerPortMapping(dsid,0,gip, 500,alg_serverips[dsid][RTL8651_ALG_IPSEC_IDX], 500);


	
	rtl8651a_delNaptServerPortMapping(dsid,0,gip, 5060,alg_serverips[dsid][RTL8651_ALG_SIP_IDX], 5060);
#if 0	
	
	rtl8651a_delNaptServerPortMapping(dsid,1,gip, 47624,alg_serverips[dsid][RTL8651_ALG_DX7_IDX], 47624);

	
	rtl8651a_delNaptServerPortRange(dsid,1,gip, 2000,alg_serverips[dsid][RTL8651_ALG_ICUII_IDX], 2000,2);
	rtl8651a_delNaptServerPortMapping(dsid,0,gip, 2002,alg_serverips[dsid][RTL8651_ALG_ICUII_IDX], 2002);

	
	rtl8651a_delNaptServerPortMapping(dsid,TRUE,gip, 6112,alg_serverips[dsid][RTL8651_ALG_BATTLENET_IDX], 6112);
	rtl8651a_delNaptServerPortMapping(dsid,FALSE,gip, 6112,alg_serverips[dsid][RTL8651_ALG_BATTLENET_IDX], 6112);	

	
	rtl8651a_delNaptServerPortMapping(dsid, FALSE, gip, TFTP_UDP_DPORT, alg_serverips[dsid][RTL8651_ALG_TFTP_IDX], TFTP_UDP_DPORT);
#endif
	return SUCCESS;	
}
	

void rtl8651_setAlgStatus(uint32 alg_support,ipaddr_t *alg_serverip)
{
	 rtl8651a_setAlgStatus(0,alg_support,alg_serverip);
}

void rtl8651_setAlgConfig(uint32 alg_support,ipaddr_t *alg_serverip,struct rtl8651_algMaxSession maxSession)
{
	 rtl8651a_setAlgConfig(0,alg_support,alg_serverip,maxSession);
}


int32 rtl8651a_setAlgQosQueueId(uint32 SessionID,uint32 alg_idx,uint32 queueID,uint32 isHigh, uint32 ratio)
{
	if(alg_idx>=32) return FAILED;
	alg_qosqid[SessionID][alg_idx]=queueID;
	alg_qoshl[SessionID][alg_idx]=isHigh;	
	alg_qosRatio[SessionID][alg_idx] = ratio;
	return SUCCESS;
}

int32 rtl8651_setAlgQosQueueId(uint32 alg_idx,uint32 queueID,uint32 isHigh, uint32 ratio)
{
	return rtl8651a_setAlgQosQueueId(0,alg_idx,queueID,isHigh, ratio);
}

int32 _rtl8651_addAlgQosNaptConnection(int8 fromDrv, int16 assigned, int8 flowType, ipaddr_t insideLocalIpAddr, uint16 insideLocalPort, 
			ipaddr_t *insideGlobalIpAddr, uint16 *insideGlobalPort, ipaddr_t dstIpAddr, uint16 dstPort,uint32 AlgIdx) 
{
	

if((assigned&ALG_QOS)&&(assigned&EXTIPPORT_ASSIGNED)&&(alg_qosqid[0][AlgIdx]!=0))
{
	rtl8651_tblDrvAclRule_t rule;

	memset(&rule, 0, sizeof(rtl8651_tblDrvAclRule_t));
	rule.actionType_ = RTL8651_ACL_DROP;

	switch(flowType) {

	case UDP_FLOW: 
		rule.ruleType_	= RTL8651_ACL_UDP;
		rule.udpSrcPortUB_	= insideLocalPort;
		rule.udpSrcPortLB_		= insideLocalPort;


#if 0		
		if(dstPort)
		{
			rule.udpDstPortUB_	= dstPort;
			rule.udpDstPortLB_		= dstPort;
		}
		else			
#endif					
		{
			rule.udpDstPortUB_	= 65535;
			rule.udpDstPortLB_		= 0;
		}
		break;

	default: 
		rule.ruleType_	= RTL8651_ACL_TCP;
		rule.tcpSrcPortUB_		= insideLocalPort;
		rule.tcpSrcPortLB_		= insideLocalPort;

#if 0
		if(dstPort)
		{
			rule.tcpDstPortUB_	= dstPort;
			rule.tcpDstPortLB_	= dstPort;
		}
		else	
#endif					
		{		
			rule.tcpDstPortUB_	= 65535;
			rule.tcpDstPortLB_	= 0;
		}
		break;		
	}
	
	rule.srcIpAddr_		= insideLocalIpAddr;
	rule.srcIpAddrMask_	= 0xffffffff;

#if 0
	if(dstIpAddr)
	{
		rule.dstIpAddr_		= dstIpAddr;
		rule.dstIpAddrMask_	= 0xffffffff;	
	}
#endif	

	{
		int r=0;
		if(alg_qoshl[0][AlgIdx])  
			r=rtl8651_addFlowToUnlimitedQueue("eth1", &rule, alg_qosRatio[0][AlgIdx], alg_qosqid[0][AlgIdx]);
		else
			r=rtl8651_addFlowToLimitedQueue("eth1", &rule, alg_qosRatio[0][AlgIdx], alg_qosqid[0][AlgIdx]);
		if(r) printk("addFlowToUn/LimiteQueue Failed , lport=%d r=%d ishigh=%d qid=%d\n",insideLocalPort,r,alg_qoshl[0][AlgIdx],alg_qosqid[0][AlgIdx]);
	}

}

return _rtl8651_addNaptConnection(fromDrv, assigned, flowType,  insideLocalIpAddr,insideLocalPort, 
			insideGlobalIpAddr, insideGlobalPort, dstIpAddr,  dstPort) ;


}


uint32 rtl8651a_setAlgStatus(uint32 SessionID,uint32 alg_support,ipaddr_t *alg_serverip)
{
	struct rtl8651_algMaxSession maxSession;	
	maxSession.ipsecMaxSession=32;
	maxSession.l2tpMaxSession=32;
	maxSession.pptpMaxSession=32;	
	rtl8651a_setAlgConfig(SessionID,alg_support,alg_serverip,maxSession);
	return SUCCESS;
}

uint32 rtl8651a_setAlgConfig(uint32 SessionID,uint32 alg_support,ipaddr_t *alg_serverip,struct rtl8651_algMaxSession maxSession)
{
	int i;
	uint32 dsid=_rtl8651_SessionIDToDSID(SessionID);
	ipaddr_t gip=_rtl8651_getExtIpAddrByDsid(dsid);
	
	if(gip==0) return NOT_OK;

	rtl8651_memcpy(&algMaxSession,&maxSession,sizeof(maxSession));
	
	alg_supports[dsid]=alg_support;

	rtl8651a_flushAlgServerIn(dsid);


	
	#define FTPALG_INSIDE_SERVER_PASSIVE_MODE

	if((alg_support&RTL8651_ALG_FTP)&&(alg_serverip[RTL8651_ALG_FTP_IDX]))
	{		
		rtl8651a_addNaptServerPortMapping(dsid,(1|2),gip, 21,alg_serverip[RTL8651_ALG_FTP_IDX], 21);	
	}

	#ifdef FTPALG_INSIDE_SERVER_PASSIVE_MODE
		rtl8651_fwdEngineRegisterAlgModule(21, TRUE,
			rtl8651_l4AliasFtpInit,
			(alg_funcptr)rtl8651_l4AliasHandleFtpCleintOutbound,
			(alg_funcptr)rtl8651_l4AliasHandleFtpCleintInbound,
			(alg_funcptr)rtl8651_l4AliasHandleFtpServerOutbound,
			NULL);
	#else
		rtl8651_fwdEngineRegisterAlgModule(21, TRUE,
			rtl8651_l4AliasFtpInit,
			(alg_funcptr)rtl8651_l4AliasHandleFtpCleintOutbound,
			NULL,
			(alg_funcptr)rtl8651_l4AliasHandleFtpServerOutbound,
			(alg_funcptr)rtl8651_l4AliasFilterFtpServerInboundPasvCmd);
	#endif	


	
	
	if((alg_support&RTL8651_ALG_NETMEETING)&&(alg_serverip[RTL8651_ALG_NETMEETING_IDX]))
	{		
		
		rtl8651a_addNaptServerPortMapping(dsid,(1|2),gip, 1720,alg_serverip[RTL8651_ALG_NETMEETING_IDX], 1720);
		rtl8651a_addNaptServerPortMapping(dsid,(1),gip, 1503,alg_serverip[RTL8651_ALG_NETMEETING_IDX], 1503);
	}
	
	rtl8651_fwdEngineRegisterAlgModule(1719, FALSE,
		rtl8651_l4AliasH323Init,
		(alg_funcptr)rtl8651_l4AliasHandleClientH225RASOut,
		(alg_funcptr)rtl8651_l4AliasHandleClientH225RASIn,
		(alg_funcptr)rtl8651_l4AliasHandleServerH225RASOut,
		(alg_funcptr)rtl8651_l4AliasHandleServerH225RASIn);

	rtl8651_fwdEngineRegisterAlgModule(1720, TRUE,
		rtl8651_l4AliasH323Init,
		(alg_funcptr)rtl8651_l4AliasHandleClientH225CSOut,
		(alg_funcptr)rtl8651_l4AliasHandleClientH225CSIn,
		(alg_funcptr)rtl8651_l4AliasHandleServerH225CSOut,
		(alg_funcptr)rtl8651_l4AliasHandleServerH225CSIn);
	

       rtl8651_fwdEngineRegisterAlgModule(17200, TRUE,
		NULL,
		NULL, 
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleServerH245Out,
		(alg_funcptr)rtl8651_l4AliasHandleServerH245In);

		
	rtl8651_fwdEngineRegisterAlgModule(1723, TRUE,
		rtl8651_l4AliasPptpInit,
		(alg_funcptr)rtl8651_l4AliasHandlePptpClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandlePptpClientInbound,
		(alg_funcptr)rtl8651_l4AliasHandlePptpServerOutbound,
		(alg_funcptr)rtl8651_l4AliasHandlePptpServerInbound);

		
	if((alg_support&RTL8651_ALG_PPTP_SERVER)&&(alg_serverip[RTL8651_ALG_PPTP_IDX]))
	{		
		
		rtl8651a_addNaptServerPortMapping(dsid,(1|2),gip, 1723,alg_serverip[RTL8651_ALG_PPTP_IDX], 1723);
	}

	
	rtl8651_fwdEngineRegisterAlgModule(500, FALSE,
		rtl8651_l4AliasIpsecInit,
		(alg_funcptr)rtl8651_l4AliasHandleIpsecClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleIpsecClientInbound,
		NULL,
		NULL);

		
	if((alg_support&RTL8651_ALG_IPSEC_SERVER)&&(alg_serverip[RTL8651_ALG_IPSEC_IDX]))
	{		
		rtl8651a_addNaptServerPortMapping(dsid,(2),gip, 500,alg_serverip[RTL8651_ALG_IPSEC_IDX], 500);
	}

	
	
	rtl8651_fwdEngineRegisterAlgModule(1701, FALSE,
		rtl8651_l4AliasL2tpInit,
		(alg_funcptr)rtl8651_l4AliasHandleL2tpClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleL2tpClientInbound,
		NULL,
		NULL);


	
	if((alg_support&RTL8651_ALG_SIP)&&(alg_serverip[RTL8651_ALG_SIP_IDX]))
	{		
		rtl8651a_addNaptServerPortMapping(dsid,(0|2),gip, 5060,alg_serverip[RTL8651_ALG_SIP_IDX], 5060);
	}

	rtl8651_fwdEngineRegisterAlgModule(5060, FALSE,
		rtl8651_l4AliasSipInit,
		(alg_funcptr)rtl8651_l4AliasHandleSipClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleSipClientInbound,
		(alg_funcptr)rtl8651_l4AliasHandleSipClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleSipClientInbound);	
#if 0
	


	
	if((alg_support&RTL8651_ALG_DX7)&&(alg_serverip[RTL8651_ALG_DX7_IDX]))
	{		
		rtl8651a_addNaptServerPortMapping(dsid,(1|2),gip, 47624,alg_serverip[RTL8651_ALG_DX7_IDX], 47624);
	}

	
	rtl8651_fwdEngineRegisterAlgModule(47624, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleDx7playClientOutbound, 
		NULL,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleDx7playServerInbound); 


		rtl8651_fwdEngineRegisterAlgModule(2300, TRUE,
			NULL,
			(alg_funcptr)rtl8651_l4AliasHandleDx7playClientOutbound2, 
			NULL, 
			NULL,
			(alg_funcptr)rtl8651_l4AliasHandleDx7playServerInbound2); 


		
	if((alg_support&RTL8651_ALG_ICUII)&&(alg_serverip[RTL8651_ALG_ICUII_IDX]))
	{		
		rtl8651a_addNaptServerPortRange(dsid,1,gip, 2000,alg_serverip[RTL8651_ALG_ICUII_IDX], 2000,2);
		rtl8651a_addNaptServerPortMapping(dsid,(0|2),gip, 2002,alg_serverip[RTL8651_ALG_ICUII_IDX], 2002);
	}	
#endif 

	
	rtl8651_fwdEngineRegisterAlgModule(5190, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleIcqClientOutbound,
		NULL,
		NULL,
		NULL);


	for(i=0;i<_RTL8651_ALG_ICQ_MAX_CLIENT;i++)		
		rtl8651_fwdEngineRegisterAlgModule(24500+i, TRUE,
			NULL,
			NULL,
			NULL,
			(alg_funcptr)rtl8651_l4AliasHandleIcqServerOutbound,
			NULL);

				
	rtl8651_fwdEngineRegisterAlgModule(5050, TRUE,
		rtl8651_l4AliasYmsgInit,
		(alg_funcptr)rtl8651_l4AliasHandleYmsgClientOutbound,
		NULL,
		NULL,
		NULL);
#if 0
		
	rtl8651_fwdEngineRegisterAlgModule(6667, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleIrcDccClientOutbound,
		NULL,
		NULL,
		NULL);

	rtl8651_fwdEngineRegisterAlgModule(6668, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleIrcDccClientOutbound,
		NULL,
		NULL,
		NULL);

	
	rtl8651_fwdEngineRegisterAlgModule(7000, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleVDOliveClientOutbound,
		NULL,
		NULL,
		NULL);

	
	rtl8651_fwdEngineRegisterAlgModule(27910, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleQuakeClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleQuakeClientInbound,
		NULL,
		NULL);	

	
	rtl8651_fwdEngineRegisterAlgModule(27960, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleQuakeClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleQuakeClientInbound,
		NULL,
		NULL);

	
	rtl8651_fwdEngineRegisterAlgModule(6112, TRUE,
		NULL,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleBattlenetClientInbound,
		NULL,
		NULL);

	
	if((alg_support&RTL8651_ALG_BATTLENET)&&(alg_serverip[RTL8651_ALG_BATTLENET_IDX]))
	{		
		rtl8651a_addNaptServerPortMapping(dsid,(FALSE|PURE_SW),gip, 6112,alg_serverip[RTL8651_ALG_BATTLENET_IDX], 6112);
		rtl8651a_addNaptServerPortMapping(dsid,(TRUE|PURE_SW),gip, 6112,alg_serverip[RTL8651_ALG_BATTLENET_IDX], 6112);		
	}	
#endif 
	
	rtl8651_fwdEngineRegisterAlgModule(6112, TRUE,
		rtl8651_l4AliasHandleBattlenetInit,
		(alg_funcptr)rtl8651_l4AliasHandleBattlenetClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleBattlenetClientInbound,
		NULL,
		NULL);

	
	rtl8651_fwdEngineRegisterAlgModule(7070, TRUE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleRealAudioClientOutbound,
		NULL,
		NULL,
		NULL);
#if 0
	
	rtl8651_fwdEngineRegisterAlgModule(554, TRUE,
		rtl8651_l4AliasRTSPInit,
		(alg_funcptr)rtl8651_l4AliasHandleRTSPClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleRTSPClientInbound,
		NULL,
		NULL);

	
	rtl8651_fwdEngineRegisterAlgModule(7648, FALSE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleCUseeMeClientOutbound,
		(alg_funcptr)rtl8651_l4AliasHandleCUseeMeClientInbound,
		NULL,
		NULL);		

	
	rtl8651_fwdEngineRegisterAlgModule(27015, FALSE,
		NULL,
		(alg_funcptr)rtl8651_l4AliasHandleCSClientOutbound,
		NULL,
		NULL,
		NULL);

	
	if((alg_support&RTL8651_ALG_TFTP)&&(alg_serverip[RTL8651_ALG_TFTP_IDX]))
	{		
		rtl8651a_addNaptServerPortMapping(	dsid,
											FALSE,
											gip,
											TFTP_UDP_DPORT,
											alg_serverip[RTL8651_ALG_TFTP_IDX],
											TFTP_UDP_DPORT);
	}	
	rtl8651_fwdEngineRegisterAlgModule(	TFTP_UDP_DPORT,
										FALSE,
										NULL,
										(alg_funcptr)rtl8651_l4AliasHandleTftpClientOutbound,
										NULL,
										NULL,
										NULL);

#endif 
#if 0 
	
	rtl8651_fwdEngineRegisterAlgModule(1863, TRUE,
		rtl8651_l4AliasMmsInit,
		rtl8651_l4AliasHandleMmsCleintOutbound,
		NULL,
		NULL,
		NULL);
		
		
	rtl8651_fwdEngineRegisterAlgModule(137, FALSE,
		NULL,
		rtl8651_l4AliasHandleClientUdpNbtNSOut,
		rtl8651_l4AliasHandleClientUdpNbtNSIn,
		NULL,
		NULL);		
	rtl8651_fwdEngineRegisterAlgModule(138, FALSE,
		NULL,
		rtl8651_l4AliasHandleClientUdpNbtOut,
		rtl8651_l4AliasHandleClientUdpNbtIn,
		NULL,
		NULL);	
		
		
	rtl8651_fwdEngineRegisterAlgModule(RTSP_CONTROL_PORT_NUMBER_1, TRUE,
		rtl8651_l4AliasSmediaInit,
		rtl8651_l4AliasHandleSmediaCleintOutbound,
		NULL,
		NULL,
		NULL);		
	rtl8651_fwdEngineRegisterAlgModule(RTSP_CONTROL_PORT_NUMBER_2, TRUE,
		NULL,
		rtl8651_l4AliasHandleSmediaCleintOutbound,
		NULL,
		NULL,
		NULL);	
				
#endif 

	rtl8651_memcpy(alg_serverips[dsid],alg_serverip,32*sizeof(uint32));
	return OK;

}


int32 rtl8651_algOneSecondTimer(void){
	rtl8651_l4AliasPptpHouseKeeping();
	rtl8651_l4AliasIpsecHouseKeeping();
#if 0
	rtl8651_l4AliasRtspHouseKeeping();
#endif 
	return SUCCESS;
}

void rtl8651_algReInit(void)
{
#if 0
	rtl8651_l4AliasHandleBattlenetReInit();
	rtl8651_l4AliasHandleCsReInit();
	rtl8651_l4AliasHandleDx7playReInit();
	rtl8651_l4AliasSipReInit();	
#endif
	rtl8651_l4AliasHandleBattlenetReInit();
	rtl8651_l4AliasFtpReInit();
	rtl8651_l4AliasH323ReInit();
	rtl8651_l4AliasIcqReInit();	
	rtl8651_l4AliasIpsecReInit();	
	rtl8651_l4AliasPptpReInit();	
	rtl8651_l4AliasYmsgReInit();
}



