/*
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <error.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <time.h>
#include <net/ethernet.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <dni_nvram.h>
#include <netinet/ip_icmp.h>
#include <dni_shutils.h>
#include <ctype.h>
#include <sys/sysinfo.h>
#include <sys/file.h>

#include	<net/if.h>
//#include <etsockio.h>
#include <base64encode.h>
#include <net/if_arp.h>
//#include <dni_dummy.h>
#include <pathnames.h>
#include <netdb.h>

#include <re865x.h>

#define cprintf		printf

typedef struct __rtl8651_tblDrvDemandRoute_t
{
	int type;
	union
	{
		int ip_s;
		int ip_d;
		struct {
			uint16 start ;
			uint16	end;
		}dport;
	}trigger;
	uint32 ip_start;
	uint32 ip_end;
	int identity;
	int32 (*p_handler)(uint32); 				/* call out handler */
} rtl8651_tblDrvDemandRoute_t;

#define TRIGGER_DSTIP 			0x10

void AddDemandRoute(int pppObjId,int defaultSession)
{
	rtl8651_tblDrvDemandRoute_t tmpRt;
	//int32  i;
	uint32 ip_start,ip_end;
	//uint16 port_start,port_end;
	
	rtl8651_flushDemandRoute(pppObjId);
	
	if (defaultSession){
		tmpRt.type=TRIGGER_DSTIP;
		tmpRt.ip_start=0;
		tmpRt.ip_end=0xffffffff;
		rtl8651_addDemandRoute(&tmpRt,pppObjId,0);
		return;
	}
}

/*
 * Reads file and returns contents
 * @param	fd	file descriptor
 * @return	contents of file or NULL if an error occurred
 */
char *
fd2str(int fd)
{
	char *buf = NULL;
	size_t count = 0, n;

	do {
		buf = realloc(buf, count + 512);
		if (!buf) {
			sleep(1);
			continue;
		}	
		n = read(fd, buf + count, 512);
		if (n < 0) {
			free(buf);
			buf = NULL;
		}
		count += n;
	} while (n == 512);

	close(fd);
	if (buf)
		buf[count] = '\0';
	return buf;
}

/*
 * Reads file and returns contents
 * @param	path	path to file
 * @return	contents of file or NULL if an error occurred
 */
char *
file2str(const char *path)
{
	int fd;

	if ((fd = open(path, O_RDONLY)) == -1) {
		perror(path);
		return NULL;
	}

	return fd2str(fd);
}

int str2file(char *path, char *string)
{
int fd;
int iLen;

	if( !string )
		return -1;

	iLen = strlen( string );
	
	if ((fd = open(path, O_CREAT | O_WRONLY | O_TRUNC ) )== -1)  {
		perror(path);
		return -1;
	}

	if( write( fd, string, iLen )  != iLen )
		return -1;

	close( fd );
	return iLen;
}

/* 
 * Waits for a file descriptor to change status or unblocked signal
 * @param	fd	file descriptor
 * @param	timeout	seconds to wait before timing out or 0 for no timeout
 * @return	1 if descriptor changed status or 0 if timed out or -1 on error
 */
int
waitfor(int fd, int timeout)
{
	fd_set rfds;
	struct timeval tv = { timeout, 0 };

	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);
	return select(fd + 1, &rfds, NULL, NULL, (timeout > 0) ? &tv : NULL);
}

/* 
 * Concatenates NULL-terminated list of arguments into a single
 * commmand and executes it
 * @param	argv	argument list
 * @param	path	NULL, ">output", or ">>output"
 * @param	timeout	seconds to wait before timing out or 0 for no timeout
 * @param	ppid	NULL to wait for child termination or pointer to pid
 * @return	return value of executed command or errno
 */
int
_eval(char *const argv[], char *path, int timeout, int *ppid)
{
	pid_t pid;
	int status;
	int fd;
	int flags;
	int sig;

	switch (pid = fork()) {
	case -1:	/* error */
		perror("fork");
		return errno;
	case 0:		/* child */
		/* Reset signal handlers set for parent process */
		for (sig = 0; sig < (_NSIG-1); sig++)
			signal(sig, SIG_DFL);

		/* Clean up */
		ioctl(0, TIOCNOTTY, 0);
		close(STDIN_FILENO);
		setsid();

		/* Redirect stdout to <path> */
		if (path) {
			flags = O_WRONLY | O_CREAT;
			if (!strncmp(path, ">>", 2)) {
				/* append to <path> */
				flags |= O_APPEND;
				path += 2;
			} else if (!strncmp(path, ">", 1)) {
				/* overwrite <path> */
				flags |= O_TRUNC;
				path += 1;
			}
			if ((fd = open(path, flags, 0644)) < 0)
				perror(path);
			else {
				dup2(fd, STDOUT_FILENO);
				close(fd);
			}
		}

		/* execute command */
		dprintf("%s\n", argv[0]);
		setenv("PATH", "/sbin:/bin:/usr/sbin:/usr/bin", 1);
		alarm(timeout);
		execvp(argv[0], argv);
		perror(argv[0]);
		exit(errno);
	default:	/* parent */
		if (ppid) {
			*ppid = pid;
			return 0;
		} else {
			waitpid(pid, &status, 0);
			//printf("_eval::pid[%d] status[%d]\n", pid, status);
			if (WIFEXITED(status))
				return WEXITSTATUS(status);
			else
				return status;
		}
	}
}

int vEval(char *const argv[])
{
pid_t pid;
int status;

	if((pid = vfork()) == 0)
	{
		// Child
		execvp(argv[0], argv);
		perror( argv[0] );
		exit( errno );
	}
	else if( pid == -1 )
	{
		perror("vfork");
		return errno;
	}
	else
	{
		// Parent
		waitpid(pid, &status, 0);
		if (WIFEXITED(status))
			return WEXITSTATUS(status);
		else
			return status;
	}
}

/* 
 * Kills process whose PID is stored in plaintext in pidfile
 * @param	pidfile	PID file
 * @return	0 on success and errno on failure
 */
int
kill_pidfile(char *pidfile)
{
	FILE *fp = fopen(pidfile, "r");
	char buf[256];

	if (fp && fgets(buf, sizeof(buf), fp)) {
		pid_t pid = strtoul(buf, NULL, 0);
		fclose(fp);
		return kill(pid, SIGTERM);
  	} else
		return errno;
}


/*
 * fread() with automatic retry on syscall interrupt
 * @param	ptr	location to store to
 * @param	size	size of each element of data
 * @param	nmemb	number of elements
 * @param	stream	file stream
 * @return	number of items successfully read
 */
int
safe_fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	size_t ret = 0;

	do {
		clearerr(stream);
		ret += fread((char *)ptr + (ret * size), size, nmemb - ret, stream);
	} while (ret < nmemb && ferror(stream) && errno == EINTR);

	return ret;
}

/*
 * fwrite() with automatic retry on syscall interrupt
 * @param	ptr	location to read from
 * @param	size	size of each element of data
 * @param	nmemb	number of elements
 * @param	stream	file stream
 * @return	number of items successfully written
 */
int
safe_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream)
{
	size_t ret = 0;

	do {
		clearerr(stream);
		ret += fwrite((char *)ptr + (ret * size), size, nmemb - ret, stream);
	} while (ret < nmemb && ferror(stream) && errno == EINTR);

	return ret;
}

/*
 * Convert Ethernet address string representation to binary data
 * @param	a	string in xx:xx:xx:xx:xx:xx notation
 * @param	e	binary data
 * @return	TRUE if conversion was successful and FALSE otherwise
 */
int
ether_atoe(char *a, unsigned char *e)
{
	char *c = (char *) a;
	int i = 0;

	memset(e, 0, ETHER_ADDR_LEN);
	for (;;) {
		e[i++] = (unsigned char) strtoul(c, &c, 16);
		if (!*c++ || i == ETHER_ADDR_LEN)
			break;
	}
	return (i == ETHER_ADDR_LEN);
}


/*
 * Convert Ethernet address binary data to string representation
 * @param	e	binary data
 * @param	a	string in xx:xx:xx:xx:xx:xx notation
 * @return	a
 */
char *
ether_etoa(const unsigned char *e, char *a)
{
	char *c = a;
	int i;

	for (i = 0; i < ETHER_ADDR_LEN; i++) {
		if (i)
			*c++ = ':';
		c += sprintf(c, "%02X", e[i] & 0xff);
	}
	return a;
}


char * trim(char *s)
{
	/* trim trailing whitespace */
	while (isspace(s[strlen(s)-1]))
		s[strlen(s)-1]='\0';

	/* trim leading whitespace */
	memmove(s, &s[strspn(s, " \n\r\t\v")], strlen(s));
	return s;
}


static const int MAXIPLEN = 60;

static int in_cksum(unsigned short *buf, int sz)
{
	int nleft = sz;
	int sum = 0;
	unsigned short *w = buf;
	unsigned short ans = 0;

	while (nleft > 1) {
		sum += *w++;
		nleft -= 2;
	}

	if (nleft == 1) {
		*(unsigned char *) (&ans) = *(unsigned char *) w;
		sum += ans;
	}

	sum = (sum >> 16) + (sum & 0xFFFF);
	sum += (sum >> 16);
	ans = ~sum;
	return (ans);
}

int ping( char *strPingAddr, int iSec, int iUsec, int iTryTimes)
{
	struct sockaddr_in pingaddr;
	struct sockaddr_in from;
	int iFromLen;
	
	struct icmp *pkt;
	int pingsock;
	int iBytes;
	int i;
	char packet[ MAXIPLEN ];
	fd_set		fdset;
	struct timeval 	tm;
	time_t	now;

	if( !strPingAddr ) return -1;
	
	if ((pingsock = socket( AF_INET, SOCK_RAW, 1 )) < 0) 
	{        /* 1 == ICMP */
		cprintf("%s::socket fail\n", __FUNCTION__);
		close(pingsock);
		return (-1);
	}

	memset(&pingaddr, 0, sizeof(struct sockaddr_in));

	pingaddr.sin_family = AF_INET;
	inet_aton( strPingAddr, &pingaddr.sin_addr) ;

	pkt = (struct icmp *) packet;
	memset(pkt, 0, sizeof(packet));
	pkt->icmp_type = ICMP_ECHO;
	// Add ID and SEQ to ICMP packet to differentiate packets sent by other task
	*((time_t *) &pkt->icmp_hun.ih_idseq) = now = time( NULL );
	pkt->icmp_cksum = in_cksum((unsigned short *) pkt, sizeof(packet));
	tm.tv_sec = iSec;
	tm.tv_usec = iUsec;
	for(i=0; i<iTryTimes; i++)
	{
		iBytes = sendto(pingsock, packet, sizeof(packet), 0,
				   (struct sockaddr *) &pingaddr, sizeof(struct sockaddr_in));

		if (iBytes < 0 || iBytes != sizeof(packet))
		{
			cprintf("%s::sendto fail, ret[%d], err[%s]\n", __FUNCTION__, iBytes, strerror(errno));
			close(pingsock);
			return (-1);
		}

		FD_ZERO(&fdset);
		FD_SET(pingsock, &fdset);
		if (select(pingsock + 1, &fdset, (fd_set *) NULL, (fd_set *) NULL, &tm) < 0) 
		{
			if (errno != EINTR) 
			{
				cprintf("%s::Error on selecting: %s", __FUNCTION__, strerror(errno));
				close(pingsock);
				return (-1);
			}
		} 
		else if (FD_ISSET(pingsock, &fdset)) 
		{
			iFromLen = sizeof( from );
			if ((iBytes = recvfrom(pingsock, packet, sizeof(packet), 0,
							  (struct sockaddr *) &from, &iFromLen)) < 0) 
			{
				if (errno == EINTR)
					continue;
				else
					cprintf("%s::Error on receving: %s", __FUNCTION__, strerror(errno));
			}
			else if (iBytes >= MAXIPLEN ) 
			{	/* ip + icmp */
				struct iphdr *iphdr = (struct iphdr *) packet;

				if( pingaddr.sin_addr.s_addr != from.sin_addr.s_addr )
					continue;
				
				pkt = (struct icmp *) (packet + (iphdr->ihl << 2));	/* skip ip hdr */
				if ((pkt->icmp_type == ICMP_ECHOREPLY) &&
					( *((time_t *) &pkt->icmp_hun.ih_idseq) == now ) )
					
				{
					close(pingsock);
					return 0;	// success
				}
			}
			else
			{
				close(pingsock);
				return -1;
			}
		}
	}
	close(pingsock);
	return -1;
}

int CheckWanIfIPAddr(char *ifname)
{
	int s;
	struct ifreq ifr;
	int ret;
	
	// Open socket to kernel 
	if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
		return 0;
		
	// Check for valid IP address
	strcpy(ifr.ifr_name, ifname);
	if (ioctl(s, SIOCGIFADDR, &ifr) < 0)
		ret=0;   //this IF doesn't have IP
	else
		ret=1;  //this IF got IP
	
	close(s);
	return ret;

}


#define PING_TIMEOUT_SECS		3
#define PING_RETRY_TIMES		3
#define QRY_DNS_TIMEOUT_SECS	3
#define QRY_DNS_RETRY_TIMES	3

int g_hearbeat_is_listening=0;
int getWanState(char *ifname)
{
	char *wan_proto=nvram_safe_get("wan_proto");
	char *pStrGateWay = nvram_safe_get( "wan0_gateway" );
	FILE *fp;
	
	
	if( !strcmp(wan_proto, "static") )
	{  //only exist WAN_DOWN and WAN_CONNECTED
		if( !ping( pStrGateWay, 2, 0, 3 ) )
			return WAN_CONNECTED;
		else
			return WAN_DOWN;  // or WAN_CONNECTING ????
	}
	else if( !strcmp(wan_proto, "dhcp") )
	{
		if( CheckWanIfIPAddr("eth0") )  //check if the wan_if got an IP
		{
			if( !ping( pStrGateWay, 2, 0, 3 ) )
				return WAN_CONNECTED;
			else
				return WAN_DOWN;  // or WAN_CONNECTING ????
		}
		else  //wan_ip hasn't got ip
		{
			return WAN_DISCONNECTED;
		}
	}
	else if( !strcmp(wan_proto, "pppoe") )
	{	
		if( atoi(nvram_get("wan_pppoe_demand"))==0 )  //pppoe keepalive mode
		{
			//if( checkWanIfIPAddr("ppp0") )  //check if the wan_if(ppp0) got an IP
			if((fp = fopen("/var/run/pppd0.pid", "r"))) 
			{
				fclose(fp);
				if( !ping( pStrGateWay, 2, 0, 3 ) )
					return WAN_CONNECTED;
				else
					return WAN_DOWN;
			}
			else  //wan_ip(ppp0) hasn't got ip	
			{
				return WAN_CONNECTING;
			}
		}
		else  //pppoe dial_on_demand mode
		{
			if((fp = fopen("/var/run/pppd0.pid", "r"))) 
			{
				fclose(fp);
				
				if( !ping( pStrGateWay, 2, 0, 3 ) )
					return WAN_CONNECTED;
				else
					return WAN_DOWN;
				
				/* return WAN_CONNECTED; */
			} 
			else  //cannot open "/var/run/pppd0.pid"
			{
				return WAN_DISCONNECTED;
			}
		}
	}
	else if( !strcmp(wan_proto, "pptp") )
	{
		
		if( atoi(nvram_get("wan_pptp_idle_disconnect"))==0 )  //pptp keepalive mode
		{
			if((fp = fopen("/var/run/pppd0.pid", "r"))) 
			{
				fclose(fp);
				if( !ping( pStrGateWay, 2, 0, 3 ) )
					return WAN_CONNECTED;
				else
					return WAN_DOWN;
			}
			else  //wan_ip(ppp0) hasn't got ip	
			{
				return WAN_CONNECTING;
			}
		}
		else  //pptp dial_on_demand mode
		{
			if((fp = fopen("/var/run/pppd0.pid", "r"))) 
			{
				fclose(fp);
				
				if( !ping( pStrGateWay, 2, 0, 3 ) )
					return WAN_CONNECTED;
				else
					return WAN_DOWN;
				
				/* return WAN_CONNECTED; */
			} 
			else  //cannot open "var/run/pppd0.pid"
			{
				return WAN_DISCONNECTED;
			}
		}
	}
	else if( !strcmp(wan_proto, "l2tp") )
	{
		
		if( atoi(nvram_get("wan_l2tp_idle_disconnect"))==0 )  //pptp keepalive mode
		{
			if((fp = fopen("/var/run/pppd0.pid", "r"))) 
			{
				fclose(fp);
				if( !ping( pStrGateWay, 2, 0, 3 ) )
					return WAN_CONNECTED;
				else
					return WAN_DOWN;
			}
			else  //wan_ip(ppp0) hasn't got ip	
			{
				return WAN_CONNECTING;
			}
		}
		else  //pptp dial_on_demand mode
		{
			if((fp = fopen("/var/run/pppd0.pid", "r"))) 
			{
				fclose(fp);
				
				if( !ping( pStrGateWay, 2, 0, 3 ) )
					return WAN_CONNECTED;
				else
					return WAN_DOWN;
				
				/* return WAN_CONNECTED; */
			} 
			else  //cannot open "var/run/pppd0.pid"
			{
				return WAN_DISCONNECTED;
			}
		}
	}	
	else if( !strcmp(wan_proto, "telstra") )
	{
		if((fp = fopen(PPP_LINK_PATH"ppp0", "r"))) 
		{
			fclose(fp);
			if( !ping( pStrGateWay, 2, 0, 3 ) )
				return WAN_CONNECTED;
			else
				return WAN_DOWN;
		} 
		else  //cannot open "tmp/ppp/link.ppp0"
		{
#if 0			
			if(g_hearbeat_is_listening)
				return WAN_DISCONNECTED;
			else
				return WAN_CONNECTING;
#endif
			return WAN_DOWN;				
		}
	}

}

/**********************************
return value:
	0:	fail to open var/run/XXX.pid file
	pid: pid number
***********************************/
int get_task_pid(char *strTask)
{
	int pid; 
	char filename[80];
	char *strPid;

	
	snprintf(filename, sizeof(filename), "/var/run/%s.pid", strTask);
	if( (strPid = file2str( filename )) )
	{
		pid = atoi(strPid);
		free( strPid );
	}
	else
		pid = 0;;

	return pid;
}


/**********************************
return value:
	0:	task of this pid is dead
	1: 	task of this pid is alive
***********************************/
int pid_is_running(int pid)
{
	char filename[80];
	FILE *fp;
	
	snprintf(filename, sizeof(filename), "/proc/%d/status", pid);

	if((fp = fopen(filename, "r"))){
		//fgets(line, sizeof(line), fp);
        	/* Buffer should contain a string like "Name:   binary_name" */
		//sscanf(line, "%*s %s", name);
        	fclose(fp);
		return 1;
        }
	
	 //cprintf("cannot open %s\n", filename);
        return 0;
}


/********************************************************************
return value:
	0:	task is dead
	1:	task is alive
	2:	cannot open XXX.pid file(this task is never brought up from system start)
*********************************************************************/
int task_is_running(char *strTask)
{
	int pid;
	
	if( (pid=get_task_pid(strTask)) )
	{
		return pid_is_running(pid);
	}
	else
	{
		//cprintf("fail to open var/run/%s.pid\n", strTask);
		//return 2;
		return 0;
	}
}

unsigned long GetSysFreeMem()
{
	struct sysinfo info;

	sysinfo(&info);

	return info.freeram + info.bufferram;

}

long GetSysUpTime()
{
struct sysinfo info;

	sysinfo(&info);
	return info.uptime;
}


int getWanLinkState( int s, void /* struct ifreq */ *pIfreq, int *iSpeed )
{
	int up, speed, duplex, range;
		
	rtl8651_checkEthernetPortStatus(0/*WAN_PORT*/,&up, &speed, &duplex, &range);

	//added by dvd.chen on 01/05/2006
	//0: 10M, 1: 100M
	if(iSpeed)
		*iSpeed = (speed == 0) ? 10000000:100000000;
	//end of dvd.chen
	
	if (up)
		return LINK_UP;
	else
		return LINK_DOWN;	
}



//added by dvd.chen on 10/14/2004 
//void wan_restart( void )
void wan_restart( char *ifname )
{	
	char *argvStopWan[]= {		
		"StopWan",		
		ifname//NULL	
		};	
	char *argvStartWan[]= {		
		"StopWan",		
		"0",		
		ifname//NULL	
		};	
	int pid;

	//cprintf("doing StopWan\n");
	_eval(argvStopWan, NULL, 0, &pid);

	//Note by dvd.chen, must sleep longer enough, otherwise, the kill signal in StopWan 
	//might delayed to kiil the task restarted by StartWan
	//exetnd sleep time from 5 to 8 by dvd.chen on 07/08/2005
	//sleep(5);
	sleep(8);
	
	//cprintf("doing StartWan\n");
	_eval(argvStartWan, NULL, 0, &pid);
}
//end of dvd.chen

// Added by wayne.lu on 10/28/2004
// Get common information of the specified interface
/* Note that <netinet/in.h> defines struct in_addr
struct in_addr
{
	unsigned long s_addr;
};

All items are network order except flag argument.
For those items you don't want to access, NULL pointer is OK.
*/
int ifConfigGet
	(
	char *ifname,        				/* name of interface, i.e. ei0 */
	struct in_addr *inAddr,      		/* buffer for Internet address */
	struct in_addr *inNetmask,		/* buffer for interface netmask */
	struct in_addr *inPPPDstaAddr,		/* buffer for destination peer address */
	struct in_addr *inBroadcastAddr,	/* buffer for broadcast address */
	short int *sFlags     				/* buffer for interface flags */
	)
{
int sockfd;
struct ifreq ifreq;
int iRet = 0;

	if( !ifname )
		return -1;
	
	if ((sockfd = socket( AF_INET, SOCK_DGRAM, 0 )) < 0) 
	{
		perror("socket");
		return -1;
	}

	strncpy(ifreq.ifr_name, ifname, IFNAMSIZ);
	if( inAddr )
	{
		if ( !((iRet = ioctl(sockfd, SIOCGIFADDR, &ifreq)) < 0) ) 
			memcpy(inAddr, &(((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr), 
					sizeof(struct in_addr));
	}

	if( !(iRet < 0) && inNetmask )
	{
		if ( !((iRet = ioctl(sockfd, SIOCGIFNETMASK, &ifreq)) < 0) ) 
			memcpy(inNetmask, &(((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr), 
					sizeof(struct in_addr));
	}

	if( !(iRet < 0) && inPPPDstaAddr )
	{
		if ( !((iRet = ioctl(sockfd, SIOCGIFDSTADDR, &ifreq)) < 0) ) 
			memcpy(inPPPDstaAddr, &(((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr), 
					sizeof(struct in_addr));
	}

	if( !(iRet < 0) && inBroadcastAddr )
	{
		if ( !((iRet = ioctl(sockfd, SIOCGIFBRDADDR, &ifreq)) < 0) ) 
			memcpy(inBroadcastAddr, &(((struct sockaddr_in *) &ifreq.ifr_addr)->sin_addr), 
					sizeof(struct in_addr));
	}

	if( !(iRet < 0) && sFlags )
	{
		if ( !((iRet = ioctl(sockfd, SIOCGIFFLAGS, &ifreq)) < 0) ) 
			*sFlags = ifreq.ifr_flags;
	}

	close(sockfd);
	
	return iRet;
}


int send_signal(char *name,int id)
{

	FILE *fp;
	int pid;
	fp = fopen(name,"r");
	if (fp)
	{
		fscanf(fp,"%d",&pid);
		printf("pid1 %d\n",pid);
		kill(pid,id);
		fclose(fp);
		return 0;
	}
	return -1;
}

int check_wan_restart(void)
{

	char *wan_proto = nvram_safe_get("wan_proto");
	
	if (!strcmp(wan_proto, "dhcp") || !strcmp(wan_proto, "telstra")) {
		//send_signal("/var/run/udhcpc.pid", SIGUSR1);	/* renew */
		//sleep(3);
		return -2;
		if (getWanState("eth0") != WAN_CONNECTED)
			return -1;
	} else if (!strcmp(wan_proto, "pppoe")) {
		if( atoi(nvram_get("wan_pppoe_demand"))==0 ) { //pppoe keepalive mode
			if (getWanState("eth0") != WAN_CONNECTED) 
				return -1;
		} else if (getWanState("eth0") == WAN_DOWN)		/* dial on demand */
				return -1;
	} else if (!strcmp(wan_proto, "pptp")) {
		if( atoi(nvram_get("wan_pptp_idle_disconnect"))==0 ) { //pptp keepalive mode
			if (getWanState("eth0") != WAN_CONNECTED) 
				return -1;
		} else if (getWanState("eth0") == WAN_DOWN)		/* dial on demand */
				return -1;
	}						
	else if (!strcmp(wan_proto, "l2tp")) {
		if( atoi(nvram_get("wan_l2tp_idle_disconnect"))==0 ) { //pptp keepalive mode
			if (getWanState("eth0") != WAN_CONNECTED) 
				return -1;
		} else if (getWanState("eth0") == WAN_DOWN)		/* dial on demand */
				return -1;
	}										
	return 0;				
}		
				

/************************************************************************************
//NOTE:
//(1) the task wanna call wan_start OR wan_stop, have to make sure it can prevent zombies process
//	  the easiest way to do it is add a line at the begining of task code:
//	  signal(SIGCHLD, SIG_IGN);  //to prevent zombies process
//(2) wan_start is now only applied for pppoe, pptp, and bpalogin only
*************************************************************************************/
#define MAX_PING_TIMES	(6)
void wan_start(char *ifname)
{
	char signal[] = "XXXX";
	char *argvStartWan[]= {		
		"StopWan",		
		"0",		
		ifname//NULL	
		};
	int pid;
	int s;
	struct ifreq ifr;

	s = check_wan_restart();
	if (s != 0) {
		if (s != -2) {	/* dhcpc */
			send_signal("/var/run/boa.pid",70);		/* stop wan */
			sleep(5);
			send_signal("/var/run/boa.pid",71);		/* start wan */
		}	
	}	
	else {
		setLedLight(WAN_LED, LED_GREEN);		
	}	
	printf("%s done\n", __FUNCTION__);
}

void wan_stop(char *ifname)
{
	char *argvStopWan[]= {		
		"StopWan",		
		//NULL	
		ifname
		};
	int pid;

	//_eval(argvStopWan, NULL, 0, &pid);	
	//send_signal("/var/run/boa.pid",70);		/* stop wan */	

}

#define  MAX_RETRYS        3 
int sys_flock(int fd, int cmd)
{
    int retry=0, ret_val;
    while (retry ++ < MAX_RETRYS)
    {
       if ((ret_val =flock(fd, cmd)) < 0) 
       {
           sleep(1);
           cprintf("%s:: flock Failure =%d \n",__FUNCTION__,ret_val);
       }
       else
           break;
    }
    return (ret_val);
}

void log_WAN_connect(char *message) //Brian+: write ppp info into log file
{
	int fp;
	time_t timep;
	char buf[200];
	char cont[300];
	struct stat status;
	
	fp = open("/tmp/sys_log", O_RDWR|O_CREAT|O_APPEND); 
	if (fp == -1)
		dprintf("open file failed!!\n");
	else 
	{
		int len;

		fstat(fp, &status);
		if (status.st_size < 4096)
		{
			//flock(fp,LOCK_EX);
			sys_flock(fp,LOCK_EX); //Brian: change flock() to sys_flock()
			time(&timep);
			memset(buf,0,sizeof(buf));
			strcpy(buf, ctime(&timep));

			/* remove line feed char */
			len = strlen(buf);
			buf[len-1]='\0';

			//	sprintf(cont, "%s -WAN DHCP Client Connected IP %s\n", buf,IP);  
			sprintf(cont, "%s -%s \n", buf, message);

			write(fp, cont, strlen(cont)); 		  
			flock(fp,LOCK_UN);
		}
	} 
	close(fp);
}

uint32 checkLogFull(void)
{
	
	int fd;
	struct stat stat_buf;
	FILE *fp1, *fp2;
	int position;

	char temp[300];
	char *temp_string;
	static long last_time = 0;
	int count;
	
	//read system.log file if >16000 then log full
	fd=open("/var/system.log", O_RDONLY);
	if(fd < 0)
	{
		printf("fd open error\n");
		return 0;
	}
	
			
	fstat(fd ,&stat_buf);
	close(fd);
		
	if(stat_buf.st_size > 16000)
	{
		if (GetSysUpTime() >= (last_time + 5))
			last_time = GetSysUpTime();
		else
			return -1;	
		system("/bin/cp -f /var/system.log /var/tmp.log");
		fp1=fopen("/var/tmp.log", "r");
		if(fp1==NULL)
		{
			//printf("error in opening tmp.log\n");	
			return -1;
		}
		else
		{
                	                	
                	fp2=fopen("/var/tmp2.log", "w+");
                	
                	if(fp2 == NULL) {
                		fclose(fp1);
                		return -1;
                	}	
                
			position = 0;
			for (count = 0; count < 5; count++) { 	
                		temp_string=fgets(temp,sizeof(temp), fp1);
				temp[sizeof(temp)-1] = 0;
				if (temp_string == NULL)
					goto restore_log;

                		position += strlen(temp_string);
			}

                	fsetpos(fp1, &position);
                	
                	while( !feof(fp1))
                	{
				
                		if(fgets(temp, sizeof(temp), fp1) != NULL) {
					temp[sizeof(temp)-1] = 0;
                			fprintf(fp2, "%s", temp);                				}
                		else
                			break;

                	}	
restore_log:	
			fclose(fp2);
			fclose(fp1);
		
			rename("/var/tmp2.log","/var/system.log");
			
			unlink("/var/tmp.log");
			unlink("/var/tmp2.log");
			
			return 0;
		}		
	}
	else
		return 0;

}

int log_message(const char *fmt, ...)
{
	FILE *fp;
	va_list argptr;
	int nbytes;
	char buffer[200];	

        timer_t timep;
	struct tm *tm;
	char *wday[]={"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	char *wmonth[]={"Jan", "Feb","Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
	
	time(&timep);
	tm=gmtime(&timep);
	
	va_start(argptr, fmt);
	nbytes = vsprintf(buffer, fmt, argptr);
	va_end(argptr);
	
	if (checkLogFull() == -1) {
		return 0;
	}		

	if (!(fp = fopen("/var/system.log", "a+"))) {
			perror("write error /var/system.log");
			return -1;
	}
	
	//fputs(buffer, fp);
	if (tm->tm_year >= 105) {
		fprintf(fp, "%s,%02d %s %04d %02d:%02d:%02d %s\n",wday[tm->tm_wday], tm->tm_mday, wmonth[tm->tm_mon], (1900+tm->tm_year), tm->tm_hour, tm->tm_min, tm->tm_sec, buffer);
	}
	else {
		fprintf(fp, "%s\n", buffer);	
	}		

	fclose(fp);
	
	return 0;
}	

int arpping(u_int32_t yiaddr, u_int32_t ip, unsigned char *mac, char *interface,unsigned char *Target_mac)
{

	int	timeout = 2;
	int 	optval = 1;
	int	s;			/* socket */
	int	rv = 1;			/* return value */
	struct sockaddr addr;		/* for interface name */
	struct arpMsg	arp;
	fd_set		fdset;
	struct timeval	tm;
	time_t		prevTime;


	if ((s = socket (PF_PACKET, SOCK_PACKET, htons(ETH_P_ARP))) == -1) {
		cprintf( "Could not open raw socket\n");
		return -1;
	}
	
	if (setsockopt(s, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval)) == -1) {
//		cprintf(LOG_ERR, "Could not setsocketopt on raw socket");
		cprintf( "Could not setsocketopt on raw socket\n");
		close(s);
		return -1;
	}

	/* send arp request */
	memset(&arp, 0, sizeof(arp));
	memcpy(arp.ethhdr.h_dest, MAC_BCAST_ADDR, 6);	/* MAC DA */
	memcpy(arp.ethhdr.h_source, mac, 6);		/* MAC SA */
	arp.ethhdr.h_proto = htons(ETH_P_ARP);		/* protocol type (Ethernet) */
	arp.htype = htons(ARPHRD_ETHER);		/* hardware type */
	arp.ptype = htons(ETH_P_IP);			/* protocol type (ARP message) */
	arp.hlen = 6;					/* hardware address length */
	arp.plen = 4;					/* protocol address length */
	arp.operation = htons(ARPOP_REQUEST);		/* ARP op code */
	*((u_int *) arp.sInaddr) = ip;			/* source IP address */
	memcpy(arp.sHaddr, mac, 6);			/* source hardware address */
	*((u_int *) arp.tInaddr) = yiaddr;		/* target IP address */
	
	memset(&addr, 0, sizeof(addr));
	strcpy(addr.sa_data, interface);
	if (sendto(s, &arp, sizeof(arp), 0, &addr, sizeof(addr)) < 0)
		rv = 0;
	
	/* wait arp reply, and check it */
	tm.tv_usec = 0;
	time(&prevTime);
	while (timeout > 0) {
		FD_ZERO(&fdset);
		FD_SET(s, &fdset);
		tm.tv_sec = timeout;
		if (select(s + 1, &fdset, (fd_set *) NULL, (fd_set *) NULL, &tm) < 0) {
//			cprintf(LOG_ERR, "Error on ARPING request: %s", strerror(errno));
			if (errno != EINTR) rv = 0;
		} else if (FD_ISSET(s, &fdset)) {
			if (recv(s, &arp, sizeof(arp), 0) < 0 ) rv = 0;
			if (arp.operation == htons(ARPOP_REPLY) && 
			    bcmp(arp.tHaddr, mac, 6) == 0 && 
			    *((u_int *) arp.sInaddr) == yiaddr) {
//				cprintf(LOG_INFO, "Valid arp reply receved for this address");
				rv = 0;
				break;
			}
		}
		timeout -= time(NULL) - prevTime;
		time(&prevTime);
	}
	close(s);
	if (rv==0){
		if (Target_mac!=NULL){
			if(memcmp(arp.sHaddr,Target_mac,6)){
				rv=1;
			}
		}
	}

//	cprintf(LOG_INFO, "%salid arp replies for this address", rv ? "No v" : "V");
	cprintf("%salid arp replies for this address", rv ? "No v" : "V");
	return rv;
}


void setLedLight(int iGPIOPin, int iMode)
{
	if (iGPIOPin == TEST_LED) {
		if (iMode == LED_ON) 
			re865xIoctl("eth0",RTL8651_IOCTL_SETGPIOB0,(uint32)0,0,0);
		else
			re865xIoctl("eth0",RTL8651_IOCTL_SETGPIOB0,(uint32)1,0,0);				
	}
	else if (iGPIOPin == WAN_LED) {
		if (iMode == LED_AMBER) {
			//printf("LED Amber\n");		
			//re865xIoctl("eth0",RTL8651_IOCTL_SETGPIOB1,(uint32)1,0,0);
			re865xIoctl("eth0",RTL8651_IOCTL_SETGPIOB1,(uint32)0,0,0);			
		}	
		else {
			//printf("LED green\n");
			//re865xIoctl("eth0",RTL8651_IOCTL_SETGPIOB1,(uint32)0,0,0);
			re865xIoctl("eth0",RTL8651_IOCTL_SETGPIOB1,(uint32)1,0,0);							
		}	
	}
	
}

void system_reboot(void)
{
        re865xIoctl("eth0", RTL8651_IOCTL_REBOOT,0,0,0);
}		


#define SIOCDEVPRIVATE	0x89F0	/* to 89FF */
/*
 * re865x Ioctl (to call kernel functions including rtl8651 table driver APIs
 */
int re865xIoctl(char *name, unsigned int arg0, unsigned int arg1, unsigned int arg2, unsigned int arg3)
{
	unsigned int args[4];
	struct ifreq ifr;
	int sockfd;

	args[0] = arg0;
	args[1] = arg1;
	args[2] = arg2;
	args[3] = arg3;

	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("fatal error socket\n");
		return -3;
	}
	strcpy((char*)&ifr.ifr_name, name);
	((uint32 *)(&ifr.ifr_data))[0] = (uint32)args;
	if (ioctl(sockfd, SIOCDEVPRIVATE, &ifr)<0)
	{
		perror("device ioctl:");
		close(sockfd);
		return -1;
	}
	close(sockfd);
	return 0;
} /* end re865xIoctl */
